@@ -24,15 +24,16 @@ watiba_ref = "_watiba_"
 
 # Singleton object.
 class Compiler:
-    def __init__(self, first_stmt):
-        self.output = [first_stmt,
-                       "import watiba",
+    def __init__(self):
+        self.first_time = True
+        self.current_statement = ""
+        self.output = ["import watiba",
                        f"{watiba_ref} = watiba.Watiba()"
                        ]
         self.resolver_count = 1
         self.spawn_call = []
         self.last_stmt = ""
-        self.stmt_count = 0
+        self.stmt_count = 1
 
         # Regex expressions for Watiba commands (order matters otherwise backticks would win over spawn)
         self.expressions = {
@@ -64,9 +65,12 @@ class Compiler:
             ".*?([\-])?`(\S.*?)`.*?": self.backticks_generator
         }
 
-    # Flush out any queue spawn calls that are located after the resolver block
-    def flush(self):
-        if len(self.spawn_call) > 0:
+    # Flush output and any queue spawn calls that are located after the resolver block
+    def flush(self, final=False):
+        # Statements to ignore when looking for block terminations
+        nothingness = ["#"]
+
+        if final and len(self.spawn_call) > 0:
             if re.search("^return ", self.last_stmt.strip()):
                 # Spit out spawn calls if they're queued up
                 while len(self.spawn_call) > 0:
@@ -77,22 +81,32 @@ class Compiler:
                 print(f"      {self.last_stmt}", file=sys.stderr)
                 sys.exit(1)
 
+        # Print our generated output
+        if not self.first_time:
+            while len(self.output) > 0:
+                print(self.output.pop(0))
+
+            if len(self.current_statement.strip()) > 0:
+                self.last_stmt = self.current_statement if self.current_statement.lstrip()[
+                                                               0] not in nothingness else self.last_stmt
+
     # Generate chain command
     def chain_generator(self, parms):
         assignment = parms["match"].group(1) if parms["match"].group(1) else ""
         quote_type = "'" if "'" not in parms["match"].group(2) else '"'
-        cmd = f'{quote_type}{parms["match"].group(2)}{quote_type}' if parms["match"].group(2)[0] != "$" else parms["match"].group(2).replace("$", "")
+        cmd = f'{quote_type}{parms["match"].group(2)}{quote_type}' if parms["match"].group(2)[0] != "$" else parms[
+            "match"].group(2).replace("$", "")
         args = parms["match"].group(3)
 
-        self.output.append(f'{assignment}{watiba_ref}.chain({cmd}, {args})')
+        self.output.append(f'{parms["indentation"]}{assignment}{watiba_ref}.chain({cmd}, {args})')
 
     # Set spawn controller args
     def spawn_ctl_args(self, parms):
-        self.output.append(f'{watiba_ref}.spawn_ctlr.set_parms({parms["match"].group(1)})')
+        self.output.append(f'{parms["indentation"]}{watiba_ref}.spawn_ctlr.set_parms({parms["match"].group(1)})')
 
     # Set watiba control args
     def watiba_ctl_args(self, parms):
-        self.output.append(f'{watiba_ref}.set_parms({parms["match"].group(1)})')
+        self.output.append(f'{parms["indentation"]}{watiba_ref}.set_parms({parms["match"].group(1)})')
 
     # Handle spawn code blocks (with host specified)
     def spawn_generator_with_host(self, parms):
@@ -100,6 +114,7 @@ class Compiler:
 
     # Handle spawn code blocks
     def spawn_generator(self, parms, host=None):
+        hostname = host if host else "localhost"
         assign_idx = 1
         cmd_idx = 2
         args_idx = 3 if not host else 4
@@ -120,7 +135,7 @@ class Compiler:
         # Add in args if there's any
         resolver_args = parms["match"].group(args_idx) if parms["match"].group(args_idx) else "{}"
 
-        h = f'"{host}"' if host and host[0] != "$" else host
+        h = f'"{hostname}"' if hostname[0] != "$" else hostname
         h = h.replace("$", "") if h and h[0] == "$" else h
 
         # Queue up async call which is executed (spit out) at the end of the w_spawn block
@@ -170,9 +185,18 @@ class Compiler:
 
     # Compile the passed statement
     def compile(self, stmt):
+        # If this is the first statement to compile, keep it to generate the #! version header stuff...
+        if self.first_time:
+            self.output.insert(0, stmt)
+            self.first_time = False
+            return
+
+        # Track our current statement
+        self.current_statement = stmt
 
         # Copy the statement to a local variable
         s = str(stmt)
+        self.stmt_count += 1
 
         # Spit out spawn call if it's queued up (on block breaks)
 
@@ -188,13 +212,13 @@ class Compiler:
 
         # If done with the resolver block, did it terminate with a resolve value?
         if resolver_level_completed:
-                if re.search("^return ", self.last_stmt.strip()):
-                    print(self.spawn_call.pop())
-                else:
-                    print("ERROR: Resolver block not properly terminated with return.", file=sys.stderr)
-                    print(f"    Block at line {self.stmt_count} incorrectly terminated with:", file=sys.stderr)
-                    print(f"      {self.last_stmt}", file=sys.stderr)
-                    sys.exit(1)
+            if re.search("^return ", self.last_stmt.strip()):
+                print(self.spawn_call.pop())
+            else:
+                print("ERROR: Resolver block not properly terminated with return.", file=sys.stderr)
+                print(f"    Block at line {self.stmt_count} incorrectly terminated with:", file=sys.stderr)
+                print(f"      {self.last_stmt}", file=sys.stderr)
+                sys.exit(1)
 
         # Check the statement for a Watiba expresion
         for ex in self.expressions:
@@ -228,25 +252,17 @@ if __name__ == "__main__":
         print(f"ERROR: Input file must be type .wt.  Found {in_file}")
         sys.exit(1)
 
-    c = None
-
-    # Statements to ignore when looking for block terminations
-    nothingness = ["#"]
+    # Instantiate a compiler
+    c = Compiler()
 
+    # Read through input file and compile each statement
     with open(in_file, 'r') as f:
         for statement in f:
-            if not c:
-                c = Compiler(statement.rstrip())
-            else:
-                c.stmt_count += 1
-                c.compile(statement.rstrip())
-                for o in c.output:
-                    print(o)
-                c.output = []
-
-                if len(statement.strip()) > 0:
-                    c.last_stmt = statement if statement.lstrip()[0] not in nothingness else c.last_stmt
+            # Compile this line of input
+            c.compile(statement.rstrip())
 
+            # Spit out the output of the compiler
+            c.flush()
 
     # Flush out any queued spawn statement calls
-    c.flush()
+    c.flush(final=True)
